import pytest
import shapely
from shapely.geometry import shape


@pytest.fixture
def polygon_whitehorse():
    return shapely.from_wkt(open("tests/data/whitehorse_wkt.txt").read())


@pytest.fixture
def polygon_verdun():
    return shapely.from_wkt(open("tests/data/verdun_wkt.txt").read())


@pytest.fixture
def polygon_hole():
    return shape(
        {
            "type": "FeatureCollection",
            "name": "schefferville_demo_with_hole",
            "features": [
                {
                    "type": "Feature",
                    "properties": {
                        "name": "Schefferville (demo)",
                        "note": "Simplified test polygon with an inner ring (hole), WGS84 (EPSG:4326).",
                    },
                    "geometry": {
                        "type": "Polygon",
                        "coordinates": [
                            [
                                [-66.9000, 54.7800],
                                [-66.7500, 54.7800],
                                [-66.7500, 54.8600],
                                [-66.9000, 54.8600],
                                [-66.9000, 54.7800],
                            ],
                            [
                                [-66.8400, 54.8300],
                                [-66.8400, 54.8000],
                                [-66.8100, 54.8000],
                                [-66.8100, 54.8300],
                                [-66.8400, 54.8300],
                            ],
                        ],
                    },
                }
            ],
        }["features"][0]["geometry"]
    )



import shapely
import geohash_polygon
from polygon_geohasher.polygon_geohasher import (
    polygon_to_geohashes as polygon_to_geohashes_py,
)
import pytest


@pytest.mark.parametrize(
    "polygon, exception_message_idx",
    [
        (None, 0),
        (True, 0),
        ("string", 0),
        (1, 0),
        (1.0, 0),
        ([1, 2, 3], 0),
        ((1, 2, 3), 0),
        ({}, 0),
        (shapely.geometry.Point((-99.1795917, 19.432134)), 1),
    ],
)
def test_exception_when_invalid(polygon, exception_message_idx):
    exception_messages = [
        r"Exception while trying to extract Geometry. This function requires a Shapely Polygon or MultiPolygon.*",
        r"The geometry is not a Polygon or MultiPolygon",
    ]
    with pytest.raises(
        ValueError,
        match=exception_messages[exception_message_idx],
    ):
        geohash_polygon.polygon_to_geohashes(polygon, 3, True)


@pytest.mark.parametrize(
    "level, inner",
    [
        (1, False),
        (1, True),
        (2, False),
        (2, True),
        (3, False),
        (3, True),
        (4, False),
        (4, True),
        (5, False),
        (5, True),
        (6, False),
        (6, True),
        (7, False),
        (7, True),
    ],
)
def test_simple_polygon(level, inner):
    polygon = shapely.geometry.Polygon(
        [
            (-99.1795917, 19.432134),
            (-99.1656847, 19.429034),
            (-99.1776492, 19.414236),
            (-99.1795917, 19.432134),
        ]
    )
    assert geohash_polygon.polygon_to_geohashes(
        polygon, level, inner
    ) == polygon_to_geohashes_py(polygon, level, inner)


@pytest.mark.parametrize(
    "level, inner",
    [
        (1, False),
        (1, True),
        (2, False),
        (2, True),
        (3, False),
        (3, True),
        (4, False),
        (4, True),
        (5, False),
        (5, True),
        (6, False),
        (6, True),
        # (7, False),
        # (7, True),
    ],
)
def test_whitehorse(level, inner, polygon_whitehorse):
    assert geohash_polygon.polygon_to_geohashes(
        polygon_whitehorse, level, inner
    ) == polygon_to_geohashes_py(polygon_whitehorse, level, inner)


@pytest.mark.parametrize(
    "level, inner",
    [
        (1, False),
        (1, True),
        (2, False),
        (2, True),
        (3, False),
        (3, True),
        (4, False),
        (4, True),
        (5, False),
        (5, True),
        (6, False),
        (6, True),
        # (7, False),
        # (7, True),
    ],
)
def test_verdun(level, inner, polygon_verdun):
    assert geohash_polygon.polygon_to_geohashes(
        polygon_verdun, level, inner
    ) == polygon_to_geohashes_py(polygon_verdun, level, inner)


@pytest.mark.parametrize(
    "level, inner",
    [
        (1, False),
        (1, True),
        (2, False),
        (2, True),
        (3, False),
        (3, True),
        (4, False),
        (4, True),
        (5, False),
        (5, True),
        (6, False),
        (6, True),
        # (7, False),
        # (7, True),
    ],
)
def test_hole(level, inner, polygon_hole):
    assert geohash_polygon.polygon_to_geohashes(
        polygon_hole, level, inner
    ) == polygon_to_geohashes_py(polygon_hole, level, inner)
